

#    MIDAS Access Client  for the  DataAcquisition/Run Control service

package require MIDASAccessClient

global MIDAS_DataAcquisitionControlClient_Exists
set MIDAS_DataAcquisitionControlClient_Exists 1

global DataAcquisitionControlServerHost DataAcquisitionControlServerPort
global SOAPServerHost SOAPServerPort

if {[info exists SOAPServerHost] == 0} then {set SOAPServerHost localhost}
if {[info exists SOAPServerPort] == 0} then {set SOAPServerPort 8015}

if {[info exists DataAcquisitionControlServerHost] == 0} then {set DataAcquisitionControlServerHost $SOAPServerHost}
if {[info exists DataAcquisitionControlServerPort] == 0} then {set DataAcquisitionControlServerPort $SOAPServerPort}

variable uri "urn:DataAcquisitionControlServer"
variable proxy "http://$DataAcquisitionControlServerHost:$DataAcquisitionControlServerPort/DataAcquisitionControlServer"

global DataAcquisitionControlClient__TimeOut
global DataAcquisitionControlClient__Debug

set DataAcquisitionControlClient__TimeOut 10000; # Not enough for LyrTech setup calls (see below)
set DataAcquisitionControlClient__Debug 0

# proc insert-debug {s} {puts "$s"}

proc DataAcquisitionControlClient_Setup {args} {
   global DataAcquisitionControlServerHost DataAcquisitionControlServerPort
   global DataAcquisitionControlClient__Debug
   variable uri
   variable proxy

   set proxy "http://$DataAcquisitionControlServerHost:$DataAcquisitionControlServerPort/DataAcquisitionControlServer"

   if {[llength $args]} {set prefix [first $args]_} {set prefix ""}
   foreach create [info procs create_DataAcquisitionControlClient__*] {
     $create $prefix
     if {$DataAcquisitionControlClient__Debug} {
       set client [second [split $create _]]; set proc [last [split $create _]];
       set method [set prefix][set client]__$proc;
       if {[info procs $method] != ""} {
         insert-debug "Created $method"
       } else {
         insert-debug "Failed to create $method"
       }
     }
   }
}

proc DataAcquisitionControlClient_Configure {} {
   global  DataAcquisitionControlServerHost DataAcquisitionControlServerPort
   variable proxy

   set proxy "http://$DataAcquisitionControlServerHost:$DataAcquisitionControlServerPort/DataAcquisitionControlServer"

   foreach method [info procs DataAcquisitionControlClient__*] {catch {SOAP::configure $method -proxy $proxy}}
}

# if args is set destroy the SOAP method procs prefixed with $args_
# if args is not set destroy all SOAP method procs except those with no prefix
proc DataAcquisitionControlClient_Destroy {args} {
   global DataAcquisitionControlServerHost DataAcquisitionControlServerPort
   global DataAcquisitionControlClient__Debug
   variable uri
   variable proxy

   if {[llength $args]} {
     set prefix [first $args]_
     insert-debug "Destroying [set prefix]DataAcquisitionControlClient__*"
     foreach method [info procs [set prefix]DataAcquisitionControlClient__*] {
       rename $method ""
       if {$DataAcquisitionControlClient__Debug} {
         if {[info procs $method] == ""} {
           insert-debug "Destroyed $method"
         } else {
           insert-debug "Failed to destroy $method"
         }
       }
     }
   } else {
     insert-debug "Destroying all but DataAcquisitionControlClient__*"
     set create [info procs create_DataAcquisitionControlClient__*]
     set prefix "*_"
     foreach method [info procs [set prefix]DataAcquisitionControlClient__*] {
       if {[lsearch $create $method] == -1} {
         rename $method ""
         if {$DataAcquisitionControlClient__Debug} {
           if {[info procs $method] == ""} {
             insert-debug "Destroyed $method"
           } else {
             insert-debug "Failed to destroy $method"
           }
         }
       } else {
         if {$DataAcquisitionControlClient__Debug} {
           if {[info procs $method] == ""} {
             insert-debug "Destroyed $method in error!!"
           } else {
             insert-debug "Skipped $method"
           }
         }
       }
     }
   }
}

proc create_DataAcquisitionControlClient__GetState {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__GetState] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__GetState -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__GetState -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name GetState \
        -params {} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request
   }
}

proc create_DataAcquisitionControlClient__GetLastError {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__GetLastError] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__GetLastError -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__GetLastError -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name GetLastError \
        -params {} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request
   }
}

proc create_DataAcquisitionControlClient__GetInformation {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__GetInformation] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__GetInformation -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__GetInformation -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name GetInformation \
        -params {} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request
   }
}

proc create_DataAcquisitionControlClient__DoReset {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__DoReset] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__DoReset -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__DoReset -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name DoReset \
        -params {Parameter string} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
        -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply
   }
}

proc create_DataAcquisitionControlClient__DoSetUp {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   set TimeOut [expr $DataAcquisitionControlClient__TimeOut * 12]; # default wasn't enough for LyrTech setup calls

   if {[info procs [set prefix]DataAcquisitionControlClient__DoSetUp] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__DoSetUp -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__DoSetUp -proxy $proxy -uri $uri \
        -timeout $TimeOut \
        -name DoSetUp \
        -params {Parameter string} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
        -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply
   }
}

proc create_DataAcquisitionControlClient__DoStop {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__DoStop] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__DoStop -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__DoStop -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name DoStop \
        -params {Parameter string} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
        -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply
   }
}

proc create_DataAcquisitionControlClient__DoGo {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__DoGo] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__DoGo -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__DoGo -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name DoGo \
        -params {Parameter string} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
        -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply
   }
}

proc create_DataAcquisitionControlClient__DoOption {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__DoOption] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__DoOption -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__DoOption -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name DoOption \
        -params {Parameter string} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
        -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply
   }
}

proc create_DataAcquisitionControlClient__GetOption {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__GetOption] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__GetOption -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__GetOption -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name GetOption \
        -params {Parameter string} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request
   }
}

proc create_DataAcquisitionControlClient__GetCounters {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__GetCounters] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__GetCounters -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__GetCounters -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name GetCounters \
        -params {} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request
   }
}

proc create_DataAcquisitionControlClient__GetRates {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__GetRates] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__GetRates -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__GetRates -proxy $proxy -uri $uri \
        -timeout $DataAcquisitionControlClient__TimeOut \
        -name GetRates \
        -params {} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request
   }
}

proc create_DataAcquisitionControlClient__ServerVerbose {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__ServerVerbose] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__ServerVerbose -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__ServerVerbose -proxy $proxy -uri $uri \
        -timeout 1000 \
        -name ServerVerbose \
        -params {opt int} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
        -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 
   }
}

proc create_DataAcquisitionControlClient__ServerImplementation {prefix} {
   global DataAcquisitionControlClient__TimeOut
   variable uri
   variable proxy

   if {[info procs [set prefix]DataAcquisitionControlClient__ServerImplementation] != ""} then {
      catch {SOAP::configure [set prefix]DataAcquisitionControlClient__ServerImplementation -proxy $proxy}
   } else {
      SOAP::create [set prefix]DataAcquisitionControlClient__ServerImplementation -proxy $proxy -uri $uri \
        -timeout 1000 \
        -name ServerImplementation \
        -params {opt int} \
        -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
        -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 
   }
}

DataAcquisitionControlClient_Setup

package provide DataAcquisitionControlClient 1.0
package provide RunControlClient 1.0

puts "RunControlClient provided"

