# ap.tcl---apparatus access
#
#  17 Aug 1995  VP   Tk Migration
# Wed Nov  4 11:41:51 GMT 1992, DB
#    added user: field
# Thu Oct 22 17:16:44 BST 1992, DB
#    added password: field
# 20 Aug 92 PHO
#    implemented help widget to call manual browser

set started 0

set frame AP
set frametitle "Accessing an Apparatus"
set icontitle  "AppAccess"

set directory "$env(MIDASAPPLBASE)/apparatus"
set apparatus ""
set password ""
set resources {}
set user $env(USER)

set hdrcolour yellow


proc dirnames l {
   set ll {}
   foreach f $l {lappend ll [first $f]}
   return $ll
}

proc read-apparatus {} {
   global directory apparatus
   global resources
   set resources [:apparatus read :path $directory :name $apparatus]
}

proc read-resources {} {
   global resources
   set resources [:experiment resources]
}

proc format-resource ps {
   global apparatus
   set app $apparatus
   foreach p $ps {
      switch [first $p] {
         name {set name [second $p]}
         host {set host [second $p]}
         port {set port [second $p]}
         type {set type [second $p]}
         role {set role [second $p]}
         apparatus {set app [second $p]}
         ipaddress {set ipaddress [second $p]}
         capability {set cap [second $p]}
         flags {set flags [second $p]}
         default {}
      }
   }
   set hostport "$host/$port"
   format "%-12s%-10s%-10s%-16s%-10s" $name $role $type $hostport \
      $app
}

proc make-resource-list rs {
   set ll {}
   foreach r $rs {lappend ll [format-resource $r]}
   return $ll
}

proc click-attach {} {
   global directory apparatus
   :apparatus attach :path $directory :name $apparatus
   repaint
}

proc click-detach {} {
   global apparatus
   :apparatus detach :name $apparatus
   repaint
}

proc click-claim {} {
   global apparatus resources
   :apparatus claim :name $apparatus
   read-resources
   foreach line $resources {
      set name [second [first $line]]
      set capability [second [seventh $line]]
      if {$capability == "0x0000000000000000"} then {
            midas-warning "server $name has a null capability\nYou will be unable to access this server"
      }
   }
   paint-directory
}

proc click-free {} {
   global apparatus
   :apparatus free :name $apparatus
   paint-directory
}

proc click-inspect {} {
   clear-footer
   make-framer
   read-apparatus
   paint-apparatus
}

proc click-directory d {
   global directory
   set directory $d
   paint-directory
}

proc click-apparatus e {
   global apparatus
   set apparatus $e
}

proc click-user u {
   global user password
   clear-footer
   set user $u
#   :spectra auth :user $user :pass $password

          set z [catch {eg "(spectrum :user '$user' :pass '$password' !auth)"} m]
          if {$z != 0} then {
             display-midas-error-report self $m "Attempting to authorise spectrum access"
             error $m
          }
          return $m
}

proc click-password p {
   global user password
   clear-footer
   set password $p
#   :spectra auth :user $user :pass $password

          set z [catch {eg "(spectrum :user '$user' :pass '$password' !auth)"} m]
          if {$z != 0} then {
             display-midas-error-report self $m "Attempting to authorise spectrum access"
             error $m
          }
          return $m
}

proc repaint {} {
   global env directory apparatus user password

     :modify DIRECTORY :value "$directory"
     :modify APPARATUS :value "$apparatus"
     :modify USER      :value "$user"
     :modify PASSWORD  :value "$password"
}

proc paint-apparatus {} {
   global env resources apparatus
   if {$apparatus == ""} then {return}
   :modify [self frame-name].insp :label "Apparatus Inspection for: $apparatus"
   :modify APR-LIST :empty T :usingtag taghdr :strings "name        role        type     host/port      apparatus\n"
   foreach line [make-resource-list $resources] {
     :modify APR-LIST :strings "$line\n"
   }
   :modify APR-LIST :strings "" :see 0
}

proc paint-directory {} {
   global directory
   set dirlist [lsort [strip-dots [dirnames [:directory files :path $directory]]]]
   :modify LIST :empty T :append T :strings $dirlist :see 0
}

proc click-list {vl} {
   global COMMAND apparatus
   switch $COMMAND  {
     :list-select   {set apparatus $vl}
     :list-deselect {set apparatus ""}
   }
   repaint
}

proc click-redisplay {} {
   paint-directory
   click-password ""
   repaint
}

#    uses the default click procedure
#    default versions of start, finish, click-quit are used


set :help(USER) "This is the user id used by Claim if needed for server access"

proc make-frame {} {
   global env directory

   :frame [self frame-name] :label "[self frame-title]" :show-footer T \
    :icon-name "[self icon-title]" :resize F

   :panel INFO :padx 5 :pady 5 :layout L
    :layout V
    :row-gap 5
    :text DIRECTORY :label "Directory:" :w 24  :valuex 12c :value "$directory"
    :text APPARATUS :label "Apparatus:" :w 18  :valuex 12c
    :text USER      :label "User:"      :w 12  :valuex 12c
    :text PASSWORD  :label "Password:"  :w 12  :valuex 12c
    [:path PASSWORD].entry configure -show *
    :help PASSWORD  "This is the password used by Claim if needed for server access"

   :panel L :layout L :pady 5
   :list LIST :rows 8 :cols 18 :scroll V :help "These are the available apparatus files - select the one you want"

   :panel B1 :padx 5 :layout B :after FOOTER
   :layout H; :col-gap 10
   :button ATTACH; :button DETACH; :button CLAIM; :button FREE

   :next-row T
   :button INSPECT :label "Inspect Apparatus"
   :hskip 150
   :button HELP  :command click-help

   :show T
}

proc make-framer {} {
  global hdrcolour
   :frame [self frame-name].insp :transient T
   :layout B
   :galley APR-LIST :rows 4 :cols 64 :deaf T :font fixed
   :modify APR-LIST :createtag "taghdr -foreground $hdrcolour"
   :show T
}
