
#   pkgIndex.tcl   -  core

  global env

        switch $env(os) {
           Win   {
               set env(libDir) "bin_Win"
               set env(binDir) "bin_Win"
           }
           SunOS    {
              switch -glob $env(machine) {
                 *4u -
                 *4c -
                 *4m {
                    set env(libDir) "lib_SunOS"
                    set env(binDir) "bin_SunOS"
                 }
                 i86pc {
                    set env(libDir) "lib_SunOSx86"
                    set env(binDir) "bin_SunOSx86"
                  }
                 default {puts stderr "$env(machine) not supported"}
              }
           }
           Linux  {
              switch  $env(machine) {
                 x64_64 -
                 x686 -
                 x386 {
                    set env(libDir) "lib_Linux"
                    set env(binDir) "bin_Linux"
                 }
                 ppc {
                    if {![info exists env(processor)]} {set env(processor) ppc_4xx}
                    set env(libDir) "lib_Linux-$env(processor)"
                    set env(binDir) "bin_Linux-$env(processor)"
                 }
                 arm {
                    if {![info exists env(processor)]} {set env(processor) arm}
                    set env(libDir) "lib_Linux-$env(processor)"
                    set env(binDir) "bin_Linux-$env(processor)"
                 }
                 default {puts stderr "$env(machine) not supported"}
              }
           }
           Linux64  {
              switch -glob $env(machine) {
                 x64_64 {
                    set env(libDir) "lib_Linux64"
                    set env(binDir) "bin_Linux64"
                 }
                 default {puts stderr "$env(machine) not supported"}
              }
           }
           default  {puts stderr "$env(os) not supported"}
        }

if {![info exists env(MIDASPACKAGEWARNINGS)]} {set env(MIDASPACKAGEWARNINGS) 0}
if {![info exists env(MIDASExcludeCommonLib)]} {set env(MIDASExcludeCommonLib) 0}
if {![info exists env(MIDASExcludeSOAPLib)]} {set env(MIDASExcludeSOAPLib) 0}
if {![info exists env(MIDASExcludeSOAPServer)]} {set env(MIDASExcludeSOAPServer) 1}
if {![info exists env(MIDASExcludeMIDASLib)]} {set env(MIDASExcludeMIDASLib) 0}
if {![info exists env(MIDASExcludePackageLib)]} {set env(MIDASExcludePackageLib) 1}

proc init-packages {} {
    global env
    
    set version [info tclversion]
    
#    Common  TclTK packages
    if {[file isdir [file join $env(MIDASBASE) TclTkpackages]] && $env(MIDASExcludeCommonLib) == 0} then {

         source [file join $env(MIDASBASE) TclTkpackages package-load.tcl]
         init-packages_[set version]
     }

#    SOAP client commands
    if {[file isdir [file join $env(MIDASBASE) SOAP tcl]] && $env(MIDASExcludeSOAPLib) == 0} then {

         source [file join $env(MIDASBASE) SOAP tcl pkgIndex.tcl]
     }

#    SOAP server implementation
    if {[file isdir [file join $env(MIDASBASE) TclHttpd tcl]] && $env(MIDASExcludeSOAPServer) == 0} then {

         source [file join $env(MIDASBASE) TclHttpd tcl pkgIndex.tcl]
     }

#    MIDAS specific packages

    if {[file isdir [file join $env(MIDASBASE) MIDASpackages]] && $env(MIDASExcludeMIDASLib) == 0} then {

         source [file join $env(MIDASBASE) MIDASpackages pkgIndex.tcl]
    }

#    Finally try for an additional package-load.tcl file 

    if {[file isdir [file join $env(MIDASBASE) packages]] && $env(MIDASExcludePackageLib) == 0} then {

         source [file join $env(MIDASBASE) packages package-load.tcl]
         init-packages_[set version]
    }
}


